import regex
from recognizers_choice.choice.extractors import BooleanExtractorConfiguration
from recognizers_choice.resources.english_choice import EnglishChoice
from recognizers_text.utilities import RegExpUtility
from typing import Pattern


class EnglishBooleanExtractorConfiguration(BooleanExtractorConfiguration):
    regex_true: Pattern
    regex_false: Pattern
    token_regex: Pattern
    only_top_match: bool

    def __init__(self, only_top_match: bool = True):
        self.regex_true = RegExpUtility.get_safe_reg_exp(
            EnglishChoice.TrueRegex)
        self.regex_false = RegExpUtility.get_safe_reg_exp(
            EnglishChoice.FalseRegex)
        self.token_regex = RegExpUtility.get_safe_reg_exp(
            EnglishChoice.TokenizerRegex, regex.S)
        self.only_top_match = only_top_match



from typing import Dict, List

from recognizers_choice.choice.extractors import ChoiceExtractDataResult
from recognizers_choice.choice.constants import Constants
from recognizers_text.extractor import ExtractResult
from recognizers_text.parser import Parser, ParseResult


class ChoiceParseDataResult:
    score: float
    other_matches: List[ExtractResult]

    def __init__(self, score=0.0, other_matches=[]):
        self.score = score
        self.other_matches = other_matches


class OtherMatchParseResult:
    score: float
    text: str
    value: object

    def __init__(self, score, text, value):
        self.score = score
        self.text = text
        self.value = value


class ChoiceParserConfiguration:
    resolutions: Dict[str, object]


class ChoiceParser(Parser):
    config: ChoiceParserConfiguration

    def __init__(self, config: ChoiceParserConfiguration):
        self.config = config

    def parse(self, ext_result):
        result = ParseResult(ext_result)
        data = ChoiceExtractDataResult(ext_result.data)
        result.value = self.config.resolutions.get(result.type)
        result.data = ChoiceParseDataResult(
            data.score, [self.__to_other_match_result(m) for m in data.other_matches])
        return result

    def __to_other_match_result(self, ext_result):
        parse_result = ParseResult(ext_result)
        ext_data = ChoiceExtractDataResult(ext_result.Data)

        result = OtherMatchParseResult(ext_data.score,
                                       parse_result.text,
                                       self.config.resolutions.get(parse_result.type))
        return result


class BooleanParser(ChoiceParser):
    def __init__(self):
        res: Dict[str, bool] = {
            Constants.SYS_BOOLEAN_TRUE: True,
            Constants.SYS_BOOLEAN_FALSE: False,
        }

        config = ChoiceParserConfiguration()
        config.resolutions = res

        ChoiceParser.__init__(self, config)



from enum import IntFlag
from typing import List

from recognizers_choice.choice.extractors import BooleanExtractor
from recognizers_choice.choice.english import EnglishBooleanExtractorConfiguration
from recognizers_choice.choice.models import BooleanModel
from recognizers_choice.choice.parsers import BooleanParser
from recognizers_text import Culture, Recognizer, ModelResult, Model


class ChoiceOptions(IntFlag):
    NONE = 0


def recognize_boolean(query: str,
                      culture: str,
                      options: ChoiceOptions = ChoiceOptions.NONE,
                      fallback_to_default_culture: bool = True) -> List[ModelResult]:
    recognizer = ChoiceRecognizer(culture, options)
    model = recognizer.get_boolean_model(culture, fallback_to_default_culture)
    return model.parse(query)


class ChoiceRecognizer (Recognizer[ChoiceOptions]):

    def __init__(self, target_culture: str = None, options: ChoiceOptions = ChoiceOptions.NONE, lazy_initialization: bool = False):
        if options < ChoiceOptions.NONE or options > ChoiceOptions.NONE:
            raise ValueError()
        super().__init__(target_culture, options, lazy_initialization)

    def initialize_configuration(self):
        self.register_model('BooleanModel', Culture.English, lambda options: BooleanModel(
            BooleanParser(), BooleanExtractor(EnglishBooleanExtractorConfiguration())))

    @staticmethod
    def is_valid_option(options: int) -> bool:
        return options >= 0 & options <= ChoiceOptions.NONE

    def get_boolean_model(self, culture: str = None, fallback_to_default_culture: bool = True) -> Model:
        return self.get_model('BooleanModel', culture, fallback_to_default_culture)
